@@ -36,6 +36,7 @@
 #include <fcntl.h>
 #include <memory>
 #include <string>
+#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -178,6 +179,42 @@ TEST_F(DBusTest, thumbnail_no_such_file)
     EXPECT_TRUE(boost::contains(message, " No such file or directory: ")) << message;
 }
 
+TEST_F(DBusTest, thumbnail_for_fd)
+{
+    // We do this twice, so we get a cache hit on the second try.
+    for (int i = 0; i < 2; ++i)
+    {
+        const char* filename = TESTDATADIR "/testvideo.ogg";
+
+        QFile file(filename);
+        ASSERT_TRUE(file.open(QIODevice::ReadOnly));
+        QDBusUnixFileDescriptor fd(file.handle());
+
+        QDBusReply<QByteArray> reply =
+            dbus_->thumbnailer_->GetThumbnailForFD(fd, QSize(256, 256));
+        assert_no_error(reply);
+
+        Image image(reply.value());
+        EXPECT_EQ(256, image.width());
+        EXPECT_EQ(144, image.height());
+    }
+}
+
+TEST_F(DBusTest, thumbnail_for_socket_fd_fails)
+{
+    int sv[2];
+
+    ASSERT_EQ(socketpair(AF_UNIX, SOCK_STREAM, 0, sv), 0);
+
+    QDBusUnixFileDescriptor fd(dup(sv[0]));
+    QDBusReply<QByteArray> reply =
+            dbus_->thumbnailer_->GetThumbnailForFD(fd, QSize(256, 256));
+
+    EXPECT_FALSE(reply.isValid());
+    auto message = reply.error().message().toStdString();
+    EXPECT_TRUE(boost::contains(message, " No such file or directory")) << message;
+}
+
 TEST_F(DBusTest, server_error)
 {
     {
